@@ -15,7 +15,7 @@ from autopycoin.dataset.generator import WindowGenerator
 
 from ..models import PoolNBEATS
 
-from ..utils import layer_test, check_attributes
+from ..test_utils import layer_test, check_attributes
 from ..losses import QuantileLossError
 from ..layers.nbeats_layers import GenericBlock, TrendBlock, SeasonalityBlock
 from . import (
@@ -27,6 +27,15 @@ from . import (
 from ..data import random_ts
 
 
+def check_shape(model, inputs, shape1, shape2):
+    outputs_predict = model.predict(inputs)
+    outputs_call = model(inputs)
+
+    for output in [outputs_predict, outputs_call]:
+        assert output[0].shape == shape1
+        assert output[1].shape == shape2
+
+
 def trend_weights(n_neurons, p_degree):
     return [
         np.zeros(shape=(3, 3)),
@@ -155,9 +164,10 @@ class NBEATSLayersTest(tf.test.TestCase, parameterized.TestCase):
             ),
             (
                 [
+                    1,
                     10,
-                    lambda: create_generic_nbeats(
-                        label_width=1,
+                    lambda label_width: create_generic_nbeats(
+                        label_width=label_width,
                         g_forecast_neurons=10,
                         g_backcast_neurons=10,
                         n_neurons=10,
@@ -166,7 +176,6 @@ class NBEATSLayersTest(tf.test.TestCase, parameterized.TestCase):
                         drop_rate=0.0,
                         share=True,
                     ),
-                    ["mse"],
                 ],
                 PoolNBEATS,
                 ["label_width", "n_models"],
@@ -229,15 +238,14 @@ class NBEATSLayersTest(tf.test.TestCase, parameterized.TestCase):
             kwargs={"blocks": blocks},
             input_dtype=floatx(),
             input_shape=(2, 3),
-            expected_output_shape=((None, 2), (None, 3)),
+            expected_output_shape=(
+                tf.TensorShape((None, 3)),
+                tf.TensorShape((None, 2)),
+            ),
             expected_output_dtype=[floatx(), floatx()],
             expected_output=[
+                -1 * tf.constant([12.5, 4.5, 5.5, 12.5, 4.5, 5.5,], shape=(2, 3),),
                 tf.constant([9.0, 4.5, 9.0, 4.5], shape=(2, 2)),
-                -1
-                * tf.constant(
-                    [12.0, y1 + 3 + 1, y2 + 3 + 2, 12.0, y1 + 3 + 1, y2 + 3 + 2,],
-                    shape=(2, 3),
-                ),
             ],
             custom_objects={"Stack": Stack},
         )
@@ -293,13 +301,19 @@ class NBEATSLayersTest(tf.test.TestCase, parameterized.TestCase):
             kwargs={"stacks": stacks},
             input_dtype=floatx(),
             input_shape=(2, 3),
-            expected_output_shape=((None, 2), (None, 3)),
+            expected_output_shape=(
+                tf.TensorShape((None, 3)),
+                tf.TensorShape((None, 2)),
+            ),
             expected_output_dtype=[floatx(), floatx()],
-            expected_output=tf.constant([18.0, 9.0, 18.0, 9.0], shape=(2, 2)),
+            expected_output=[
+                tf.constant([24.0, 8.0, 10.0, 24.0, 8.0, 10.0], shape=(2, 3)),
+                tf.constant([18.0, 9.0, 18.0, 9.0], shape=(2, 2)),
+            ],
         )
 
     @parameterized.parameters([(2, 3, 3, 1, [2], [3], [2], [3], 0.0,)])
-    def test_nbeats_attributes(
+    def test_nbeats_methods(
         self,
         label_width,
         input_width,
@@ -456,7 +470,7 @@ class NBEATSLayersTest(tf.test.TestCase, parameterized.TestCase):
         share,
     ):
 
-        model = create_interpretable_nbeats(
+        create_model = lambda: create_interpretable_nbeats(
             label_width=label_width,
             forecast_periods=forecast_periods,
             backcast_periods=backcast_periods,
@@ -469,6 +483,8 @@ class NBEATSLayersTest(tf.test.TestCase, parameterized.TestCase):
             share=share,
         )
 
+        model = create_model()
+
         self.assertIsInstance(model, NBEATS)
         trend_stack = model.stacks[0]
         seasonality_stack = model.stacks[1]
@@ -494,14 +510,33 @@ class NBEATSLayersTest(tf.test.TestCase, parameterized.TestCase):
             seasonality_stack.blocks[2].get_weights(),
         )
 
-        # Compare output shape with expected shape
-        outputs = model.predict(np.array([[1.0, 2.0, 3.0]]))
-        self.assertEqual(outputs.shape, (1, 2))
+        # Compare output shape with expected
+        check_shape(model, np.array([[1.0, 2.0, 3.0]]), (1, 3), (1, 2))
 
-        # Compare output shape with expected shape when quantiles = 3
-        model.compile(loss=QuantileLossError([0.1, 0.5, 0.9]))
-        outputs = model.predict(np.array([[1.0, 2.0, 3.0]]))
-        self.assertEqual(outputs.shape, (1, 2, 3))
+        model = create_model()
+
+        check_shape(
+            model,
+            np.array([[[1.0, 1.0], [2.0, 2.0], [3.0, 3.0]]]),
+            (1, 3, 2),
+            (1, 2, 2),
+        )
+
+        model = create_model()
+
+        # Compare output shape with expected when quantiles = 3
+        model.compile(loss=["mse", QuantileLossError([0.1, 0.5, 0.9])])
+        check_shape(model, np.array([[1.0, 2.0, 3.0]]), (1, 3), (1, 2, 3))
+
+        model = create_model()
+
+        model.compile(loss=["mse", QuantileLossError([0.1, 0.5, 0.9])])
+        check_shape(
+            model,
+            np.array([[[1.0, 1.0], [2.0, 2.0], [3.0, 3.0]]]),
+            (1, 3, 2),
+            (1, 2, 2, 3),
+        )
 
     @parameterized.parameters([(2, 5, 5, 5, 3, 2, 0.0, True)])
     def test_create_generic_nbeats(
@@ -516,7 +551,7 @@ class NBEATSLayersTest(tf.test.TestCase, parameterized.TestCase):
         share,
     ):
 
-        model = create_generic_nbeats(
+        create_model = lambda: create_generic_nbeats(
             label_width=label_width,
             g_forecast_neurons=g_forecast_neurons,
             g_backcast_neurons=g_backcast_neurons,
@@ -527,6 +562,8 @@ class NBEATSLayersTest(tf.test.TestCase, parameterized.TestCase):
             share=share,
         )
 
+        model = create_model()
+
         self.assertIsInstance(model, NBEATS)
         generic_stack_1 = model.stacks[0]
         generic_stack_2 = model.stacks[1]
@@ -553,77 +590,82 @@ class NBEATSLayersTest(tf.test.TestCase, parameterized.TestCase):
         )
 
         # Compare output shape with expected
-        outputs = model.predict(np.array([[1.0, 2.0, 3.0]]))
-        self.assertEqual(outputs.shape, (1, 2))
+        check_shape(model, np.array([[1.0, 2.0, 3.0]]), (1, 3), (1, 2))
 
-        # Compare output shape with expected when quantiles = 2
-        model.compile(loss=QuantileLossError([0.1, 0.5, 0.9]))
-        outputs = model.predict(np.array([[1.0, 2.0, 3.0]]))
-        self.assertEqual(outputs.shape, (1, 2, 3))
+        model = create_model()
+
+        check_shape(
+            model,
+            np.array([[[1.0, 1.0], [2.0, 2.0], [3.0, 3.0]]]),
+            (1, 3, 2),
+            (1, 2, 2),
+        )
+
+        model = create_model()
+
+        # Compare output shape with expected when quantiles = 3
+        model.compile(loss=["mse", QuantileLossError([0.1, 0.5, 0.9])])
+        check_shape(model, np.array([[1.0, 2.0, 3.0]]), (1, 3), (1, 2, 3))
+
+        model = create_model()
+
+        model.compile(loss=["mse", QuantileLossError([0.1, 0.5, 0.9])])
+        check_shape(
+            model,
+            np.array([[[1.0, 1.0], [2.0, 2.0], [3.0, 3.0]]]),
+            (1, 3, 2),
+            (1, 2, 2, 3),
+        )
 
     @parameterized.parameters(
         [
-            (2, 5, 5, 5, 3, 2, 0.0, True, tf.reduce_mean, (1, 2, 2), (1, 2)),
             (
                 2,
-                5,
-                5,
-                5,
-                3,
+                tf.reduce_mean,
+                [(1, 2, 2), (1, 2, 2), (1, 2, 2), (1, 2, 2, 2), (1, 2, 2)],
+                (1, 2),
+            ),
+            (
                 2,
-                0.0,
-                True,
                 lambda x, axis: tf.identity(x),
-                (5, 1, 2, 2),
+                [(1, 2, 2), (1, 2, 2), (1, 2, 2), (1, 2, 2, 2), (1, 2, 2)],
                 (10, 1, 2),
             ),
         ]
     )
-    def test_pool_nbeats(
-        self,
-        label_width,
-        g_forecast_neurons,
-        g_backcast_neurons,
-        n_neurons,
-        n_blocks,
-        n_stacks,
-        drop_rate,
-        share,
-        fn_agg,
-        shape,
-        shape2,
+    def test_pool_nbeats_as_model(
+        self, label_width, fn_agg, shape, shape2,
     ):
 
         data = random_ts(n_steps=30, n_variables=2)
 
         w = WindowGenerator(
-                input_width=6,
-                label_width=label_width,
-                shift=label_width,
-                valid_size=0,
-                test_size=0,
-                flat=False)
+            input_width=6,
+            label_width=label_width,
+            shift=label_width,
+            valid_size=0,
+            test_size=0,
+            flat=False,
+            preprocessing=lambda x, y: (x, (x, y)),
+        )
         w.from_array(data, input_columns=[0, 1], label_columns=[0, 1])
 
-        model = [create_generic_nbeats(
-            label_width=label_width,
-            g_forecast_neurons=g_forecast_neurons,
-            g_backcast_neurons=g_backcast_neurons,
-            n_neurons=n_neurons,
-            n_blocks=n_blocks,
-            n_stacks=n_stacks,
-            drop_rate=drop_rate,
-            share=share,
-        ) for _ in range(5)]
+        model = [
+            create_generic_nbeats(
+                label_width=label_width,
+                g_forecast_neurons=1,
+                g_backcast_neurons=1,
+                n_neurons=1,
+                n_blocks=1,
+                n_stacks=1,
+            )
+            for _ in range(5)
+        ]
 
-        qloss = QuantileLossError([0.5])
+        # Check quantiles model
+        qloss = QuantileLossError([0.2, 0.5])
 
-        model = PoolNBEATS(
-            n_models=10,
-            nbeats_models=model,
-            losses=["mse", "mae", "mape", qloss],
-            fn_agg=fn_agg,
-        )
+        model = PoolNBEATS(n_models=10, nbeats_models=model, fn_agg=fn_agg, seed=5)
 
         model.compile(
             tf.keras.optimizers.Adam(
@@ -634,51 +676,135 @@ class NBEATSLayersTest(tf.test.TestCase, parameterized.TestCase):
                 amsgrad=True,
                 name="Adam",
             ),
-            loss=model.get_pool_losses(),
+            loss=["mse", "mae", "mape", qloss],
             metrics=["mae"],
         )
 
         # Issue 13
-        model.fit(w.train)
-        output = model.predict(np.array([[[1.0, 1.0], [2.0, 2.0], [3.0, 3.0], [5.0, 5.0], [6.0, 6.0], [7.0, 7.0]]]))
+        model.fit(w.train, epochs=1)
+        # Check validation_data
+        model.fit(w.train, validation_data=w.valid, epochs=1)
+        # Check tensor inputs
+        for x, y, x_val, y_val in zip(w.train.take(1), w.valid.take(1)):
+            model.fit(x=x, y=y, validation_data=(x_val, y_val), epochs=1)
 
-        self.assertEqual(output.shape, shape)
+        output = model.predict(
+            np.array(
+                [
+                    [
+                        [1.0, 1.0],
+                        [2.0, 2.0],
+                        [3.0, 3.0],
+                        [5.0, 5.0],
+                        [6.0, 6.0],
+                        [7.0, 7.0],
+                    ]
+                ]
+            )
+        )
+
+        for o, s in zip(output[1], shape):
+            self.assertEqual(o.shape, s)
 
         for loss in ["mse", "mae", "mape", qloss]:
-            self.assertIn(loss, model.get_pool_losses())
-        model.reset_pool_losses(["mse", "msle", "mape"])
-        for loss in ["mse", "msle", "mape"]:
-            self.assertIn(loss, model.get_pool_losses())
+            self.assertIn(loss, model.loss)
 
-        model2 = lambda: create_generic_nbeats(
-            label_width=label_width,
-            g_forecast_neurons=g_forecast_neurons,
-            g_backcast_neurons=g_backcast_neurons,
-            n_neurons=n_neurons,
-            n_blocks=n_blocks,
-            n_stacks=n_stacks,
-            drop_rate=drop_rate,
-            share=share,
-        )
-        model2 = PoolNBEATS(
-            n_models=10,
-            nbeats_models=model2,
-            losses=["mse", "mae", "mape"],
-            fn_agg=fn_agg,
-        )
-
-        model2.compile(
-            tf.keras.optimizers.Adam(
-                learning_rate=0.02,
-                beta_1=0.9,
-                beta_2=0.999,
-                epsilon=1e-07,
-                amsgrad=True,
-                name="Adam",
+        # callable model
+        nbeats = [
+            lambda label_width: create_generic_nbeats(
+                label_width=label_width,
+                g_forecast_neurons=1,
+                g_backcast_neurons=1,
+                n_neurons=1,
+                n_blocks=1,
+                n_stacks=1,
             ),
-            loss=model.get_pool_losses(),
-            metrics=["mae"],
-        )
+            lambda label_width: create_interpretable_nbeats(
+                label_width=label_width, trend_n_neurons=1, seasonality_n_neurons=1
+            ),
+        ]
+
+        # Check randomness
+        tf.random.set_seed(5)
+
+        for i in range(2):
+            model2 = PoolNBEATS(
+                label_width=label_width,
+                n_models=10,
+                nbeats_models=nbeats,
+                fn_agg=fn_agg,
+                seed=5,
+            )
+
+            model2.compile(
+                tf.keras.optimizers.Adam(
+                    learning_rate=0.02,
+                    beta_1=0.9,
+                    beta_2=0.999,
+                    epsilon=1e-07,
+                    amsgrad=True,
+                    name="Adam",
+                ),
+                loss=["mse", "mae", "mape"],
+                metrics=["mae"],
+            )
+
+            model2.build(tf.TensorShape((None, 6)))
+
+            if i == 0:
+                mask_random_v1 = model2._mask
+                pool_losses_random_v1 = model2.compiled_loss._losses
+                types_random_v1 = [m.nbeats_type for m in model2.models]
+
+        self.assertAllEqual(model2._mask, mask_random_v1)
+        self.assertAllEqual(model2.compiled_loss._losses, pool_losses_random_v1)
+        self.assertAllEqual([m.nbeats_type for m in model2.models], types_random_v1)
+
         output = model2.predict(np.array([[1.0, 2.0, 3.0, 5.0, 6.0, 7.0]]))
 
-        self.assertEqual(output.shape, shape2)
+        self.assertEqual(output[1].shape, shape2)
+
+    def test_pool_nbeats_as_layer(self):
+
+        nbeats = [
+            lambda label_width: create_generic_nbeats(
+                label_width=label_width,
+                g_forecast_neurons=1,
+                g_backcast_neurons=1,
+                n_neurons=1,
+                n_blocks=1,
+                n_stacks=1,
+            ),
+            lambda label_width: create_interpretable_nbeats(
+                label_width=label_width, trend_n_neurons=1, seasonality_n_neurons=1
+            ),
+        ]
+
+        kwargs_2 = {"label_width": 3, "n_models": 3, "nbeats_models": nbeats, "seed": 0}
+
+        layer_test(
+            PoolNBEATS,
+            kwargs=kwargs_2,
+            input_dtype=floatx(),
+            input_shape=(2, 10),
+            expected_output_shape=(
+                (tf.TensorShape((None, 10)), tf.TensorShape((None, 3))),
+                (tf.TensorShape((None, 10)), tf.TensorShape((None, 3))),
+                (tf.TensorShape((None, 10)), tf.TensorShape((None, 3))),
+            ),
+            expected_output_dtype=[
+                floatx(),
+                floatx(),
+                floatx(),
+                floatx(),
+                floatx(),
+                floatx(),
+            ],
+        )
+
+    def test_pool_nbeats_raises_error(self):
+        with self.assertRaisesRegexp(
+            AssertionError,
+            f"""When `models` are callable, `label_width` and `n_models` have to be integers.""",
+        ):
+            PoolNBEATS(n_models=10,)
