@@ -1,4 +1,5 @@
-""" This subpackage defines all metrics needed to train a deep learning.
+""" 
+This subpackage defines helper functions needed to train a deep learning.
 """
 
 from .data_utils import (
@@ -9,21 +10,20 @@ from .data_utils import (
     convert_to_list,
     transpose_first_to_last,
     transpose_last_to_first,
+    transpose_first_to_second_last,
     features,
     date_features,
 )
-from .testing_utils import layer_test, check_attributes
 
 __ALL__ = [
     "convert_to_list",
     "example_handler",
-    "layer_test",
     "quantiles_handler",
     "range_dims",
-    "check_attributes",
     "fill_none",
     "transpose_first_to_last",
     "transpose_last_to_first",
+    "transpose_first_to_second_last",
     "features",
     "date_features",
 ]
